'use strict';

exports.__esModule = true;
exports.observable = exports.polyfill = exports.withGetApp = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.connect = connect;
exports.watchUpdate = watchUpdate;
exports.computable = computable;
exports.compose = compose;

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var withGetApp = exports.withGetApp = function withGetApp(wrappedPage) {
    return Object.assign(wrappedPage, {
        getApp: getApp
    });
};

function connect(selector) {
    return function (wrappedPage) {
        var page = Object.assign({}, wrappedPage);
        var connectSub = void 0;

        page.onLoad = function onLoad() {
            var _this = this;

            var app = getApp();
            var didUpdate$ = app.store$.map(function (x) {
                return selector(x);
            }).distinctUntilChanged(function (x, y) {
                return Object.keys(x).every(function (k) {
                    return x[k] === y[k];
                });
            });
            connectSub = didUpdate$.subscribe(function (x) {
                return _this.state$ ? _this.state$.next(x) : _this.setData(x);
            });

            for (var _len = arguments.length, params = Array(_len), _key = 0; _key < _len; _key++) {
                params[_key] = arguments[_key];
            }

            apply(this, wrappedPage, 'onLoad', params);
        };

        page.onUnload = function onUnload() {
            tryUnsubcribe(connectSub);

            for (var _len2 = arguments.length, params = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                params[_key2] = arguments[_key2];
            }

            apply(this, wrappedPage, 'onUnload', params);
        };
        return page;
    };
}

var dump = function dump() {};
var highOrderDump = function highOrderDump() {
    return dump;
};
function watchUpdate(wrappedPage) {
    var page = Object.assign({}, wrappedPage);
    page.state$ = new _Rx2.default.Subject();

    page.onLoad = function onLoad() {
        var _this2 = this;

        if (!this.state$ || this.state$.closed) {
            this.state$ = new _Rx2.default.Subject();
        }
        var didUpdate = this.didUpdate ? function (oldVal) {
            return function () {
                return _this2.didUpdate(oldVal);
            };
        } : highOrderDump;

        this.state$.map(function (x) {
            return _this2.willUpdate ? _this2.willUpdate(Object.assign({}, _this2.data, x)) || x : x;
        }).subscribe(function (x) {
            var data = Object.assign({}, _this2.data);
            _this2.setData(x, didUpdate(data));
        });

        for (var _len3 = arguments.length, params = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
            params[_key3] = arguments[_key3];
        }

        apply(this, wrappedPage, 'onLoad', params);
    };

    page.onUnload = function onUnload() {
        tryUnsubcribe(this.state$);

        for (var _len4 = arguments.length, params = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
            params[_key4] = arguments[_key4];
        }

        apply(this, wrappedPage, 'onUnload', params);
    };

    return page;
}

// 加入 computed 属性，必须前置 watchUpdate
function computable(wrappedPage) {
    var page = Object.assign({}, wrappedPage);
    page.data = Object.assign(page.data, compute(page));
    page.willUpdate = function willUpdate(nextState) {
        var next = Object.assign({}, nextState, compute(this));
        return apply(this, wrappedPage, 'willUpdate', [next]);
    };
    return page;
}

function compose(funcs) {
    if (funcs.length === 0) {
        return function (arg) {
            return arg;
        };
    }

    if (funcs.length === 1) {
        return funcs[0];
    }

    return funcs.reduce(function (a, b) {
        return function () {
            return a(b.apply(undefined, arguments));
        };
    });
}

function tryUnsubcribe(subscription) {
    if (subscription) {
        subscription.unsubcribe();
    }
}

function apply(ctx, target, funcName, args) {
    if (target[funcName]) {
        target[funcName].apply(ctx, args);
    }
}

function compute(page) {
    return page.computed ? Object.keys(page.computed).reduce(function (x, k) {
        var _extends2;

        return _extends({}, x, (_extends2 = {}, _extends2[k] = page.computed[k].apply(page), _extends2));
    }, {}) : {};
}

var polyfill = exports.polyfill = compose([withGetApp]);

var observable = exports.observable = compose([computable, watchUpdate]);
//# sourceMappingURL=index.js.map